
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "httpd.h"
#include "rfm12.h"

//-----------------------------------------------------------------------------------------

void rfm12_init()
{

  RFM12_PORT |= (1<<RFM12_IRQ) | (1<<RFM12_CS) | (1<<RFM12_SDO);    // PullUp ein / Ausgang H
  RFM12_DDR  |= (1<<RFM12_CS) | (1<<RFM12_SDI) | (1<<RFM12_SCK);    // Ausgnge
  RFM12_DDR  &= ~((1<<RFM12_IRQ) | (1<<RFM12_SDO));                 // Eingang

  MCUCR |= (1<<ISC01);                          // INT0 fallende Flanke fr RFM12 nIRQ
  GICR |= (1<<INT0);

  rfm12_send_cmd(0x0000);  // Status read

  rfm12_send_cmd(0xC080);  // CLK-Frequenz / Batterie-Level
  rfm12_send_cmd(0x80D7);  // FIFO ein, 433MHZ-Band, C = 12pF
  rfm12_send_cmd(0xC2AB);  // Clock Recovery Auto, Filter digital, DQD-Thresold 3
  rfm12_send_cmd(0xCA81);  // FIFO-Level 8 Bit, Sync Pattern ein, High senity Reset aus
  rfm12_send_cmd(0xE000);  // WakeUp aus
  rfm12_send_cmd(0xC800);  // Low Duty Cycle aus
  rfm12_send_cmd(0xC4F3);  // AFC immer, +7,5 / -10kHz, Add Freq-Offset zur PLL, Berechne Freq. Offset aus AFC                      // Status read
  rfm12_send_cmd(0xA620);  // Frequenz 433,92MHz  // Status read
  rfm12_send_cmd(0x948C);  // VDI Output, VDI Fast, Bandbreite 200kHz, LBA-Gain -6dB, RSSI-Thresold -79dB
  rfm12_send_cmd(0xC610);  // Baudrate 19200
  rfm12_send_cmd(0x8281);  // Empfnger ein, Clock Out aus
  rfm12_send_cmd(0xCA81);  // set FIFO mode
  rfm12_send_cmd(0xCA83);  // enable FIFO
}

//-----------------------------------------------------------------------------------------

uint16_t rfm12_send_cmd(uint16_t command)
{
	uint8_t i;

	RFM12_PORT &= ~(1<<RFM12_CS);			      	    // CS auf L

	for (i=0; i<16; i++)
		{
			if (command & 0x8000)                     // MSB testen und senden
			  {
			  	RFM12_PORT |= (1<<RFM12_SDI);         // DATA auf H
			  }
			else
			  {
			  	RFM12_PORT &= ~(1<<RFM12_SDI);		    // DATA auf L
			  }
		  RFM12_PORT |= (1<<RFM12_SCK);			        // CLK auf H

			command = (command << 1);					        // MSB raus schieben, Status LSB wird nach oben geschoben

			if ((RFM12_PIN & (1<<RFM12_SDO)))	        // Bit 1?
		 	  {
		  		command = command | 0x0001;	  		    // ja, LSB von Status setzen
		 	  }
	  	RFM12_PORT &= ~(1<<RFM12_SCK);		  	    // CLK auf L
		}
	RFM12_PORT |= (1<<RFM12_CS);					        // CS auf H

  return command;
}

//-----------------------------------------------------------------------------------------

ISR(INT0_vect)
{
  static uint8_t daten[32];
  static uint8_t checksum = 0;
  static uint8_t bytecount = 0;
  uint8_t i, sensor_nr, byte;      

  RFM12_PORT &= ~(1<<RFM12_CS);			            // CS auf L
 					
  byte = 0xB0;							                    // Receive FIFO

  for (i=0; i<8; i++)                           // Kommando senden
    { 
      if (byte & 0x80)
        {
			  	RFM12_PORT |= (1<<RFM12_SDI);	        // DATA auf H
			  }
			else
			  {
			 		RFM12_PORT &= ~(1<<RFM12_SDI);	      // DATA auf L
			  }
			RFM12_PORT |= (1<<RFM12_SCK);		          // CLK auf H

			byte = (byte << 1);				                // nchstes Bit nach oben

			RFM12_PORT &= ~(1<<RFM12_SCK);		        // CLK auf L
		}

  byte = 0;

	for (i=0; i<8; i++)                           // FIFO Daten lesen
    { 		
			byte = (byte << 1);					              // eins hher schieben
			if ((RFM12_PIN & (1<<RFM12_SDO)))	        // Bit 1?
		 	  {
		  		byte = (byte | 0x01);				          // ja
		 	  }
		 	RFM12_PORT |= (1<<RFM12_SCK);		          // CLK auf H
			asm("nop");
		 	RFM12_PORT &= ~(1<<RFM12_SCK);		        // CLK auf L
		}
 
	RFM12_PORT |= (1<<RFM12_CS);		        	    // CS auf H

  daten[bytecount] = byte;                      // Daten speichern
  bytecount++;
  checksum ^= byte;                             // XOR-Prfsumme

  if (bytecount == PAKET_LEN)                   // fertig?
    {
      rfm12_send_cmd(0xCA81);                   // set FIFO mode
      rfm12_send_cmd(0xCA83);                   // enable FIFO

      if (!(checksum))                          // Checksumme ok?
        {
          sensor_nr = (daten[0] & 0x0F) - 1;    // jetzt ab 0
          if (sensor_nr < SENSOR_ANZ)           // im gltigen Beeich?
            {
              daten[bytecount-1] = 1;               // 1 ins letzte Byte (Checksumme) fr neue Daten
              memcpy((void *) sensor[sensor_nr], (void const *) daten, PAKET_LEN);
            }
          daten[bytecount-1] = 1;               // 1 ins letzte Byte (Checksumme) fr neue Daten
          memcpy((void *) sensor[sensor_nr], (void const *) daten, PAKET_LEN);
        }  
      checksum = 0;
      bytecount = 0;     
    }

}

//-----------------------------------------------------------------------------------------

void rfm12_set_var()
{
  static unsigned char sensor_var[] PROGMEM = {0,2,4,7,9,10};
  unsigned char var_nr, i;

  for (i=0; i<SENSOR_ANZ; i++)
    { 
      cli();
      
      var_nr = pgm_read_byte(&sensor_var[i]);             // Nummer der ersten Variablen des Sensors holen        

      if (sensor[i][0])                                   // Daten da?
        {
          if (sensor[i][2] == 0x51)                       // Temperatur da?
            {
              var_array[var_nr][0] = sensor[i][3];        // Vorzeichen
              if (sensor[i][4] == '0')                    // fhrende Null ausblenden
                {
                  var_array[var_nr][1] = ' ';
                }
              else
                {
                  var_array[var_nr][1] = sensor[i][4];    // Zehner
                }
              var_array[var_nr][2] = sensor[i][5];        // Einer
              var_array[var_nr][3] = sensor[i][6];        // Komma
              var_array[var_nr][4] = sensor[i][7];        // Zehntel
              var_array[var_nr][5] = ' ';                 // Space
              var_array[var_nr][6] = '&';                 // html Grad
              var_array[var_nr][7] = 'd';
              var_array[var_nr][8] = 'e';
              var_array[var_nr][9] = 'g';
              var_array[var_nr][10] = ';';
              var_array[var_nr][11] = 'C';
              var_array[var_nr][12] = 0;                  // Stringende
            }
          
          if (sensor[i][9] == 0x52)                       // Feuchte da?
            {
              var_array[var_nr+1][0] = sensor[i][11];      // Zehner
              var_array[var_nr+1][1] = sensor[i][12];      // Einer
              var_array[var_nr+1][2] = ' ';               // Space
              var_array[var_nr+1][3] = '%';               // Prozent
              var_array[var_nr+1][4] = 0;                 // Stringende
            }     

          if (sensor[i][16] == 0x53)                      // Luftdruck da?
            {
              var_array[var_nr+2][0] = ' ';               // Space

              if (sensor[i][17] == '0')                   // fhrende Null ausblenden
                {
                  var_array[var_nr+2][1] = ' ';
                }
              else
                {
                  var_array[var_nr+2][1] = sensor[i][17];     // Tausender
                }
              var_array[var_nr+2][2] = sensor[i][18];     // Hunderter
              var_array[var_nr+2][3] = sensor[i][19];     // Zehner
              var_array[var_nr+2][4] = sensor[i][20];     // Einer
              var_array[var_nr+2][5] = ' ';               // Space
              var_array[var_nr+2][6] = 'h';
              var_array[var_nr+2][7] = 'P';
              var_array[var_nr+2][8] = 'a';
              var_array[var_nr+2][9] = 0;                   // Stringende
            }
        } 
      sei();
    }
}


